
#include "asteroids.h"
#include "collision.h"
#include <cstdio>

bool intersectCircleHorLine(vec2f c, float r, float x)
{
	if (x < c.x - r)
		return false;
	else if (c.x + r < x)
		return false;
	else 
		return true;
}

bool intersectCircleVerLine(vec2f c, float r, float y)
{
	if (y < c.y - r)
		return false;
	else if (c.y + r < y)
		return false;
	else 
		return true;
}

bool intersectCircleCircle(vec2f c1, float r1, vec2f c2, float r2)
{
	vec2f d;
	float dSq;

	d = c2 - c1;
	dSq = d.sizesq();

	if (dSq > (r1 + r2) * (r1 + r2))
		return false;
	else 
		return true;
}

float reactCircleHorLine(vec2f* c, vec2f* v, float r, float x)
{
	float a1F1 = r * v->size();
	if(x == 0.0)
		c -> x = x + r;
	else
		c -> x = x - r;
	v->x = -v->x;
	
	float a1F2 = r * v->size();
	
	return a1F2 - a1F1;
}

float reactCircleVerLine(vec2f* c, vec2f* v, float r, float y)
{
	float a1F1 = r * v->size();
	if(y == 0.0)
		c -> y = y + r;
	else
		c -> y = y - r;
	v->y = -v->y;
	
	float a1F2 = r * v->size();
	
	return a1F2 - a1F1;
}

float reactAsteroids(asteroid_t &a1, asteroid_t &a2, bool debug)
{	
	float a1F1 = a1.r * a1.v.size();
	
	if(debug) {
		float a1kin = 0.5*a1.r * a1.v.sizesq();
		float a2kin = 0.5*a2.r * a2.v.sizesq();
		printf("Collision: %f - ",a1kin+a2kin);
	}
	
	a2.p += (a1.p - a2.p).unit() * ( ((a1.p - a2.p).size() - (a1.r + a2.r))/2 );
	
	vec2f normal = a1.p - a2.p;
	normal.normalize();
	
	float v1i = a1.v.dot(normal);
	float v2i = a2.v.dot(normal);
	
	float mdiff,msum,twom2;
	mdiff = a1.r - a2.r;
	msum = a1.r + a2.r;
	twom2 = 2 * a2.r;
	
	float v1f = (mdiff * v1i / msum) + (twom2 * v2i / msum);
	
	mdiff = a2.r - a1.r;
	twom2 = 2 * a1.r;
	float v2f = (mdiff * v2i / msum) + (twom2 * v1i / msum);
	
	a1.v = a1.v - (normal * a1.v.dot(normal)) + (normal * v1f);
	a2.v = a2.v - (normal * a2.v.dot(normal)) + (normal * v2f);
	
	if(debug) {	
		float a1kin = 0.5*a1.r * a1.v.sizesq();
		float a2kin = 0.5*a2.r * a2.v.sizesq();
		printf("Post Collision: %f\n",a1kin+a2kin);
	}
	
	float a1F2 = a1.r * a1.v.size();
	
	float dFa1 = a1F1 - a1F2;
	return dFa1;
}

bool intersect(asteroid_t asteroid, vec2f levelBounds)
{
	return (asteroid.p - vec2f(levelBounds.x/2,levelBounds.y/2)).size() >= abs(asteroid.r - levelBounds.y/2);
}

bool intersect(ship_t ship, vec2f levelBounds)
{
	return (ship.p - vec2f(levelBounds.x/2,levelBounds.y/2)).size() >= abs(ship.size - levelBounds.y/2);
}

void react(asteroid_t &asteroid, vec2f levelBounds)
{
	vec2f normal = asteroid.p - vec2f(levelBounds.x/2,levelBounds.y/2);
	normal.normalize();
	
	asteroid.p -= normal * (asteroid.r/10);
	
	vec2f reflection = asteroid.v - (vec2f)(2*asteroid.v.dot(normal))*normal;

	asteroid.v = reflection;
}

void react(ship_t &ship, vec2f levelBounds)
{
	vec2f normal = ship.p - vec2f(levelBounds.x/2,levelBounds.y/2);
	normal.normalize();
	
	ship.p -= normal * (ship.size/10);
	
	vec2f reflection = ship.v - (vec2f)(2*ship.v.dot(normal))*normal;

	ship.v = reflection;
}
