
#include "effects.h"

const float pi = acos(-1.0);

Engine::Engine(){
	count = 0;
}

void Engine::drawPoints(){
	int i;
	glPushAttrib(GL_CURRENT_BIT);
	glBegin(GL_POINTS);
	for(i = 0; i < count; i++){
		#ifdef COLOUR
		glColor3f(RAND_UNIT+0.5,RAND_UNIT-0.25,0);
		#endif
		glVertex2f(points[i].Pos.x, points[i].Pos.y);
	}
	glEnd();
	glPopAttrib();
}

void Engine::updatePoints(float dt){
	int i;
	
	for(i = 0; i < count; i++){
		//change position
		points[i].Pos.x += points[i].Vel.x/(10000.0);
		points[i].Pos.y += points[i].Vel.y/(10000.0);
		//calculate life
		points[i].life -= dt*20;
		if(points[i].life < 0.0){
			//delete if life is too low
			deletePoint(i);
		}
	}
}

void Engine::createPoint(vec2f posSet, float heading){
	//const float pi = acos(-1.0);
	Particle tempPart;
	if(count >= cMax)
		return;
	
	//does calculations to determin location at the back of the ship
	float val = (200 * RAND_UNIT) + 20;
	if(RAND_UNIT > 0.5)
		val *= -1;
	tempPart.Pos.x = posSet.x + cosf(heading-pi/2)/10 + cosf(heading)/val;
	tempPart.Pos.y = posSet.y + sinf(heading-pi/2)/10 + sinf(heading)/val;
	tempPart.Vel.x = cosf(heading-pi/2)/10 + cosf(heading)/val;
	tempPart.Vel.y = sinf(heading-pi/2)/10 + sinf(heading)/val;
	
	//gives the particles a random lifespan
	tempPart.life = (0.5 + RAND_UNIT) * 5;
	
	points.push_back(tempPart);
	count ++;
}

void Engine::deletePoint(int i){
	points.erase(points.begin()+i);
	count --;
}
void Engine::erase(){
	points.clear();
	count = 0;
}


//EXPLOSION STUFF
Explosion::Explosion(){
	count = 0;
}

void Explosion::explode(vec2f posSet){
	int i;
	//const float pi = acos(-1.0);
	for(i = 0; i < 50; i++){
		createPoint(posSet, RAND_UNIT*(2*pi));
	}
}

void Explosion::drawPoints(){
	int i;
	
	glPushAttrib(GL_CURRENT_BIT);
	glBegin(GL_POINTS);
	for(i = 0; i < count; i++){
		#ifdef COLOUR
		float random = RAND_UNIT;
		glColor3f(random*0.4+0.3,random*0.2+0.2,0);
		#endif
		glVertex2f(points[i].Pos.x, points[i].Pos.y);
	}
	glEnd();
	glPopAttrib();
}

void Explosion::updatePoints(float dt){
	int i;
	
	for(i = 0; i < count; i++){
		points[i].Pos.x += points[i].Vel.x/(1.0f)*dt;
		points[i].Pos.y += points[i].Vel.y/(1.0f)*dt;
		points[i].life -= dt*20;
		if(points[i].life < 0.0){
			deletePoint(i);
		}
	}
}

void Explosion::createPoint(vec2f posSet, float heading){
	Particle tempPart;
	if(count >= cMax)
		return;
	
	//creates the point near the position explode was called at
	tempPart.Pos.x = posSet.x + cosf(RAND_UNIT*heading)/50;
	tempPart.Pos.y = posSet.y + sinf(RAND_UNIT*heading)/50;
	//const float pi = acos(-1.0);
	float random = RAND_UNIT;
	float random2 = RAND_UNIT;
	tempPart.Vel.x = cosf(random*2*pi)/(random2*100);
	tempPart.Vel.y = sinf(random*2*pi)/(random2*100);
	
	tempPart.life = (0.5 + RAND_UNIT) * 40;
	
	points.push_back(tempPart);
	count ++;
}

void Explosion::deletePoint(int i){
	points.erase(points.begin()+i);
	count --;
}
void Explosion::erase(){
	points.clear();
	count = 0;
}



//PLASMA STUFF
Plasma::Plasma(){
	count = 0;
}
Plasma::~Plasma(){
	erase();
}

void Plasma::drawPoints(){
	int i;
	
	glPushAttrib(GL_CURRENT_BIT);
	glBegin(GL_POINTS);
	for(i = 0; i < count; i++){
		#ifdef COLOUR
		float random = RAND_UNIT+0.2;
		glColor3f(random*0.5,0,random);
		#endif
		glVertex2f(points[i].Pos.x, points[i].Pos.y);
	}
	glEnd();
	glPopAttrib();
}

void Plasma::updatePoints(float dt){
	int i;
	
	for(i = 0; i < count; i++){
		points[i].Pos.x += points[i].Vel.x/(1.0)*dt;
		points[i].Pos.y += points[i].Vel.y/(1.0)*dt;
		points[i].life -= dt*20;
		if(points[i].life < 0.0){
			deletePoint(i);
		}
	}
}

void Plasma::createPoint(vec2f posSet, float heading){
	Particle tempPart;
	if(count >= cMax)
		return;
	
	float val = (200 * RAND_UNIT) + 20;
	if(RAND_UNIT > 0.5)
		val *= -1;
	tempPart.Pos.x = 0;
	tempPart.Pos.y = 0;
	//const float pi = acos(-1.0);
	float random = RAND_UNIT;
	float random2 = RAND_UNIT;
	tempPart.Vel.x = cosf(random*2*pi)/(random2*100)*5;
	tempPart.Vel.y = sinf(random*2*pi)/(random2*100)*5;
	
	tempPart.life = RAND_UNIT*5;
	
	points.push_back(tempPart);
	count ++;
}

void Plasma::deletePoint(int i){
	points.erase(points.begin()+i);
	count --;
}
void Plasma::erase(){
	points.clear();
	count = 0;
}




//STAR STUFF
//PLASMA STUFF
Star::Star(){
	count = 0;
}
Star::~Star(){
	erase();
}

void Star::createStars(){
	for(int i = 0; i < 50; i++){
		createPoint();
	}
}

void Star::drawPoints(){
	int i;
	
	glPushAttrib(GL_CURRENT_BIT);
	
	for(i = 0; i < count; i++){
		glPointSize(points[i].size);
		glBegin(GL_POINTS);
		#ifdef COLOUR
		float random = RAND_UNIT+0.25;
		glColor3f(random,random,random);
		#endif
		glVertex2f(points[i].Pos.x, points[i].Pos.y);
		glEnd();
	}
	glPointSize(1);
	glPopAttrib();
}

void Star::createPoint(){
	starParticle tempPart;
	if(count >= cMax)
		return;
	
	tempPart.Pos.x = RAND_UNIT * levelSize.x;
	tempPart.Pos.y = RAND_UNIT * levelSize.y;
	tempPart.size = ceil(RAND_UNIT*2);
	
	points.push_back(tempPart);
	count ++;
}

void Star::deletePoint(int i){
	points.erase(points.begin()+i);
	count --;
}
void Star::erase(){
	points.clear();
	count = 0;
}
void Star::setSize(vec2f boundsSet){
	levelSize = boundsSet;
}



//SHIP EXPLOSION STUFF
shipExplosion::shipExplosion(){
	count = 0;
}

void shipExplosion::explode(vec2f posSet){
	int i;
	//const float pi = acos(-1.0);
	for(i = 0; i < 50; i++){
		createPoint(posSet, RAND_UNIT*(2*pi));
	}
}

void shipExplosion::drawPoints(){
	int i;
	glPushAttrib(GL_CURRENT_BIT);
	glBegin(GL_POINTS);
	for(i = 0; i < count; i++){
		#ifdef COLOUR
		glColor3f(RAND_UNIT-0.5,RAND_UNIT+0.5,0);
		#endif
		glVertex2f(points[i].Pos.x, points[i].Pos.y);
	}
	glEnd();
	glPopAttrib();
}

void shipExplosion::updatePoints(float dt){
	int i;
	
	for(i = 0; i < count; i++){
		points[i].Pos.x += points[i].Vel.x/(1.0)*dt;
		points[i].Pos.y += points[i].Vel.y/(1.0)*dt;
		points[i].life -= dt*20;
		if(points[i].life < 0.0){
			deletePoint(i);
		}
	}
}

void shipExplosion::createPoint(vec2f posSet, float heading){
	Particle tempPart;
	if(count >= cMax*2)
		return;
	
	//creates the point near the position explode was called at
	tempPart.Pos.x = posSet.x + cosf(RAND_UNIT*heading)/50;
	tempPart.Pos.y = posSet.y + sinf(RAND_UNIT*heading)/50;
	//const float pi = acos(-1.0);
	float random = RAND_UNIT;
	tempPart.Vel.x = cosf(random*2*pi)/10;
	tempPart.Vel.y = sinf(random*2*pi)/10;
	
	tempPart.life = (0.5 + RAND_UNIT) * 40;
	
	points.push_back(tempPart);
	count ++;
}

void shipExplosion::deletePoint(int i){
	points.erase(points.begin()+i);
	count --;
}
void shipExplosion::erase(){
	points.clear();
	count = 0;
}
